@@ -12,6 +12,7 @@ import attr
 import yaml
 from openlineage.client.filter import Filter, FilterConfig, create_filter
 from openlineage.client.serde import Serde
+from openlineage.client.tags import TagsConfig
 from openlineage.client.utils import deep_merge_dicts
 
 if TYPE_CHECKING:
@@ -26,6 +27,8 @@ from openlineage.client.generated.environment_variables_run import (
     EnvironmentVariable,
     EnvironmentVariablesRunFacet,
 )
+from openlineage.client.generated.tags_job import TagsJobFacet, TagsJobFacetFields
+from openlineage.client.generated.tags_run import TagsRunFacet, TagsRunFacetFields
 from openlineage.client.run import DatasetEvent, JobEvent, RunEvent
 from openlineage.client.transport import (
     Transport,
@@ -53,6 +56,7 @@ class OpenLineageConfig:
     transport: dict[str, Any] | None = attr.ib(factory=dict)
     facets: FacetsConfig = attr.ib(factory=FacetsConfig)
     filters: list[FilterConfig] = attr.ib(factory=list)
+    tags: TagsConfig = attr.ib(factory=TagsConfig)
 
     @classmethod
     def from_dict(cls, params: dict[str, Any]) -> OpenLineageConfig:
@@ -63,6 +67,13 @@ class OpenLineageConfig:
             config.facets = FacetsConfig(**params["facets"])
         if "filters" in params:
             config.filters = [FilterConfig(**filter_config) for filter_config in params["filters"]]
+        if "tags" in params:
+            job_tags = params["tags"].get("job", {})
+            run_tags = params["tags"].get("run", {})
+            config.tags = TagsConfig(
+                job=[TagsJobFacetFields(key, value, "USER") for (key, value) in job_tags.items()],
+                run=[TagsRunFacetFields(key, value, "USER") for (key, value) in run_tags.items()],
+            )
         return config
 
 
@@ -163,6 +174,7 @@ class OpenLineageClient:
             return
 
         event = self.add_environment_facets(event)
+        event = self.update_event_tags_facets(event)
 
         if log.isEnabledFor(logging.DEBUG):
             val = Serde.to_json(event).encode("utf-8")
@@ -362,7 +374,6 @@ class OpenLineageClient:
             # Parse value (try to parse as JSON, otherwise lowercase the value)
             with contextlib.suppress(json.JSONDecodeError):
                 env_value = json.loads(env_value)  # noqa: PLW2901
-
             cls._insert_into_config(config, keys, env_value)
 
         return config
@@ -404,3 +415,50 @@ class OpenLineageClient:
                 missing_vars,
             )
         return filtered_vars
+
+    def update_event_tags_facets(self, event: Event) -> Event:
+        """
+        Creates or updates job and run tag facets based on user-supplied environment variables
+        """
+        tags_job = self.config.tags.job
+        if (isinstance(event, (JobEvent, RunEvent))) and tags_job:
+            tags_facet = event.job.facets.get("tags", TagsJobFacet())
+            event.job.facets["tags"] = self._update_tag_facet(tags_facet, tags_job)  # type: ignore [arg-type]
+
+        tags_run = self.config.tags.run
+        if (isinstance(event, RunEvent)) and tags_run:
+            tags_facet = event.run.facets.get("tags", TagsRunFacet())
+            event.run.facets["tags"] = self._update_tag_facet(tags_facet, tags_run)  # type: ignore [arg-type]
+
+        return event
+
+    def _update_tag_facet(
+        self,
+        tags_facet: TagsJobFacet | TagsRunFacet,
+        user_tags: list[TagsJobFacetFields | TagsRunFacetFields],
+    ) -> TagsJobFacet | TagsRunFacet:
+        """
+        Handles updating tags in an existing tag facet
+        """
+
+        # Get tags from the facet that will not be updated (Do not have the same key as a user tag)
+        user_tag_keys = [tag.key for tag in user_tags]
+        keep_tags = []
+        if tags_facet.tags is not None:
+            keep_tags = [tag for tag in tags_facet.tags if tag.key.lower() not in user_tag_keys]
+
+        # Update tag key for any user tags that should override a facet tag.
+        # This preserves case of the facet tag key while updating value and source.
+        facet_tag_keys = {}
+        if tags_facet.tags is not None:
+            facet_tag_keys = {tag.key.lower(): tag.key for tag in tags_facet.tags}
+
+        for user_tag in user_tags:
+            if user_tag.key in facet_tag_keys:
+                facet_tag_key = facet_tag_keys[user_tag.key]
+                log.info("Overriding integration-supplied tag `%s` with user-supplied tag", facet_tag_key)
+                user_tag.key = facet_tag_key
+
+        all_tags = keep_tags + user_tags
+        tags_facet.tags = all_tags  # type: ignore [assignment]
+        return tags_facet
